import sys


def get_input_file(args):
    """
    Return an input file handle

    Args:
        args (Namespace): Usually the output of parser.parse_args()
    """
    if args.input_file == "-":
        filehandle = sys.stdin
    else:
        filehandle = open(args.input_file, "r")
    return filehandle


def get_output_file(args):
    """
    Return an output file handle

    Args:
        args (Namespace): Usually the output of parser.parse_args()
    """
    if args.output_file == "-":
        filehandle = sys.stdout
    else:
        filehandle = open(args.output_file, "w")
    return filehandle



import random


def seq_permutation(seq: str, k: int = 1) -> str:
    """Shuffle a genomic sequence

    Args:
        seq (str): Sequence to be shuffled.
        k (int): For `k==1`, we shuffle individual characters. For `k>1`, we shuffle k-mers.

    Returns:
        Shuffled sequence.
    """
    if k == 1:
        return ''.join(random.sample(seq, len(seq)))
    else:
        kmers = [seq[i:(i + k)] for i in range(0, len(seq), k)]
        return ''.join(random.sample(kmers, len(kmers)))



import numpy as np
from typing import Iterable, Dict, List


ACTG = {'A': 0, 'C': 1, 'T': 2, 'G': 3}


def one_hot_encoding(seq_list: Iterable[str], alphabet: Dict[str, int] = ACTG) -> np.array:
    """One-hot encoding of a list of genomic sequences

    The sequnces must have the same length. They are encoded into a boolean numpy array.
    `X[i,j,k]` is True if the `j`th character of the sequence `i` is `k`letter in the alphabet.

    Args:
        seq_list: List of sequences. All must have the same length.
        alphabet: Dictionary mapping letters of alphabet to index.

    Returns:
        3D numpy array (number of sequences x length of one sequence x sixe of the alphabet)
    """

    n = len(seq_list)
    m = len(seq_list[0])
    k = len(alphabet.keys())
    seq_array = np.zeros((n, m, k), dtype=np.bool)

    for i in range(n):
        assert len(seq_list[i]) == m, "Sequences must be of the same length."
        for j in range(m):
            seq_array[i, j, alphabet[seq_list[i][j]]] = True
    
    return seq_array


def one_hot_decoding(seq_array: np.array, alphabet: Dict[str, int] = ACTG) -> List[str]:
    """One-hot decoding

    The function decodes boolean numpy array, ussually the output of `one_hot_encoding`.

    Args:
        seq_list: 3D numpy array (number of sequences x length x sixe of the alphabet).
        alphabet: Dictionary mapping letters of alphabet to index.

    Returns:
        List of genomic sequences.
    """

    alphabet_array = np.array(list(alphabet.keys()), dtype=str)
    seq_list = []
    n, m, _ = seq_array.shape
 
    for i in range(n):
        s = ''
        for j in range(m):
            c = alphabet_array[seq_array[i][j]][0]
            s += c
        seq_list.append(s)

    return seq_list



import numpy as np
import pandas as pd
from typing import Union, Dict, Tuple


CHR_LENGTHS_Homo_sapiens_GRCh38 = {'1': 248956422,
                                   '2': 242193529,
                                   '3': 198295559,
                                   '4': 190214555,
                                   '5': 181538259,
                                   '6': 170805979,
                                   '7': 159345973,
                                   '8': 145138636,
                                   '9': 138394717,
                                   '10': 133797422,
                                   '11': 135086622,
                                   '12': 133275309,
                                   '13': 114364328,
                                   '14': 107043718,
                                   '15': 101991189,
                                   '16': 90338345,
                                   '17': 83257441,
                                   '18': 80373285,
                                   '19': 58617616,
                                   '20': 64444167,
                                   '21': 46709983,
                                   '22': 50818468,
                                   'X': 156040895,
                                   'Y': 57227415,
                                   'MT': 16569}
CHR_LENGTHS_Homo_sapiens_GRCh38_SERIES = pd.Series(CHR_LENGTHS_Homo_sapiens_GRCh38)


def random_genomic_position(chrom_lengths: Union[Dict[str, int], pd.Series] = CHR_LENGTHS_Homo_sapiens_GRCh38_SERIES) -> Tuple[str, int]:
    """Returns a random genomic position

    The chomosome is generated from a multinomial distribution with chr. lengths used as weights. The generated position is 1-based.

    Args:
        chrom_lengths: Either Dict of pd.Series with chromosomes' lengths.

    Returns:
        A tuple of chromosome and a 1-based position on this chromosome.
    """

    chrom_lengths = pd.Series(chrom_lengths) if isinstance(chrom_lengths, dict) else chrom_lengths
    
    chrom = _random_chr(chrom_lengths)
    chrom_len = chrom_lengths[chrom]
    pos = np.random.randint(chrom_len) + 1

    return chrom, pos


def random_genomic_interval(interval_length: int,
                            chrom_lengths: Union[Dict[str, int], pd.Series] = CHR_LENGTHS_Homo_sapiens_GRCh38_SERIES) -> Tuple[str, int, int]:
    """Returns a random genomic interval

    Args:
        interval_length (int): Length of interval to be generated.
        chrom_lengths: Either Dict of pd.Series with chromosomes' lengths.

    Returns:
        A tuple of chromosome, start and end (of interval).
    """
    chrom_lengths = pd.Series(chrom_lengths) if isinstance(chrom_lengths, dict) else chrom_lengths
    chrom_lengths = (chrom_lengths - interval_length + 1).clip(0)

    chrom, start = random_genomic_position(chrom_lengths)
    end = start + interval_length - 1
    
    return chrom, start, end


def _random_chr(chrom_lengths: pd.Series):
    chrom_names = chrom_lengths.index.values
    chrom_probs = chrom_lengths / chrom_lengths.sum()
    random_chrom_index = np.argwhere(np.random.multinomial(1, chrom_probs))[0][0]

    return chrom_names[random_chrom_index]



import sys
from setuptools.command.test import test as TestCommand
from setuptools import setup, find_packages


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to pytest")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = ''

    def run_tests(self):
        import shlex
        import pytest
        errno = pytest.main(shlex.split(self.pytest_args))
        sys.exit(errno)


setup(
    name='rbp',
    version='0.1.1',
    description='Python utils of RBP Bioinformatics.',
    author='RBP Bioinformatics',
    author_email='lamparna@gmail.com',
    license='MIT',
    keywords=['bioinformatics', 'genome', 'dna'],
    url='https://gitlab.com/RBP_Bioinformatics/rbp',
    download_url='https://gitlab.com/RBP_Bioinformatics/rbp/-/archive/master/rbp-master.tar.gz',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    scripts=[],
    setup_requires=['pytest-runner'],
    install_requires=[
        'pip==19.2.2',
        'pytest>=4.6.1',
        'codecov>=2.0.15',
        'pytest-cov>=2.6.1',
        'numpy>=1.17.0',
        'pandas>=1.0.0',
        'pyaml>=18.11.0',
        'seaborn>=0.9.0',
        'matplotlib>=3.1.1',
        'tqdm>=4.42.1',
    ],
    tests_require=['pytest'],
    cmdclass={'test': PyTest},
    include_package_data=True,
    package_data={},
    classifiers=[
        'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',      # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',   # Again, pick a license
        'Programming Language :: Python :: 3',      # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



from argparse import ArgumentParser


def get_argument_parser(input_option=True, output_option=True, verbose_option=True):
    """Return argparser.ArgumentParser() with defaults used at RBP Bioinformatics

    Args:
        input_option (bool): Whether to include '--input_file' option, or not
        output_option (bool): Whether to include '--output_file' option, or not
        verbose_option (bool): Whether to include '--verbose' and '--quiet' options, or not
    """
    parser = ArgumentParser()

    if input_option:
        parser.add_argument('--input_file', '-i', type=str, help='Input file, ommit for STDIN', default='-')
    if output_option:
        parser.add_argument('--output_file', '-o', type=str, help='Output file, ommit for STDOUT', default='-')
    if verbose_option:
        parser.add_argument('--verbose', '-v', help='Make lots of noise [default]', dest='verbose', action='store_true')
        parser.add_argument('--quiet', '-q', help='Be vewwy quiet (I\'m hunting wabbits)', dest='verbose', action='store_false')
        parser.set_defaults(verbose=True)

    return parser
